@@ -6,7 +6,7 @@
 
 main( void)
 {
-    fputs( "started\n", stderr);
+    fputs( "started\n", stdout);
 
 /* 36.1:    0x5c in multi-byte character is not an escape character */
 
@@ -14,7 +14,7 @@ main( void)
 #pragma setlocale( "jis")                   /* For MCPP on VC   */
 
 #if     '$B;z(B' == '\x3b\x7a' && '$B0\(B' != '\x30\x5c'
-    fputs( "Bad handling of '\\' in multi-byte character", stderr);
+    fputs( "Bad handling of '\\' in multi-byte character", stdout);
     exit( 1);
 #endif
 
@@ -24,7 +24,10 @@ main( void)
     assert( strcmp( str( "$B1"M[(B"), "\"$B1"M[(B\"") == 0);
     assert( strcmp( str( "$B1'Ch(B"), "\"$B1'Ch(B\"") == 0);
 
-    fputs( "success\n", stderr);
+    fputs( "$B0\F0(B" "\"$B0\F0(B\"\n", stdout);
+    fputs( "$B1"M[(B" "\"$B1"M[(B\"\n", stdout);
+    fputs( "$B1'Ch(B" "\"$B1'Ch(B\"\n", stdout);
+    fputs( "success\n", stdout);
     return  0;
 }
 
